@@ -22,8 +22,10 @@ from scrapy.http.headers import Headers
 from scrapy.responsetypes import responsetypes
 from scrapy.utils.defer import deferred_from_coro
 from scrapy.utils.misc import load_object
+from scrapy.utils.python import to_unicode
 from scrapy.utils.reactor import verify_installed_reactor
 from twisted.internet.defer import Deferred, inlineCallbacks
+from w3lib.encoding import html_body_declared_encoding, http_content_type_encoding
 
 from scrapy_playwright.headers import use_scrapy_headers
 from scrapy_playwright.page import PageCoroutine
@@ -141,9 +143,10 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
             context = await self._create_browser_context(context_name, context_kwargs)
             self.contexts[context_name] = context
         page = await context.new_page()
-        page.on("response", _make_response_logger(context_name))
         page.on("request", _make_request_logger(context_name))
+        page.on("response", _make_response_logger(context_name))
         page.on("request", self._increment_request_stats)
+        page.on("response", self._increment_response_stats)
         self.stats.inc_value("playwright/page_count")
         if self.default_navigation_timeout is not None:
             page.set_default_navigation_timeout(self.default_navigation_timeout)
@@ -219,7 +222,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
                 pc.result = await method(*pc.args, **pc.kwargs)
                 await page.wait_for_load_state(timeout=self.default_navigation_timeout)
 
-        body = (await page.content()).encode("utf8")
+        body_str = await page.content()
         request.meta["download_latency"] = time() - start_time
 
         if request.meta.get("playwright_include_page"):
@@ -230,6 +233,8 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
 
         headers = Headers(response.headers)
         headers.pop("Content-Encoding", None)
+        encoding = _get_response_encoding(headers, body_str) or "utf-8"
+        body = body_str.encode(encoding)
         respcls = responsetypes.from_args(headers=headers, url=page.url, body=body)
         return respcls(
             url=page.url,
@@ -238,6 +243,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
             body=body,
             request=request,
             flags=["playwright"],
+            encoding=encoding,
         )
 
     def _increment_request_stats(self, request: PlaywrightRequest) -> None:
@@ -248,6 +254,12 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
         if request.is_navigation_request():
             self.stats.inc_value(f"{stats_prefix}/navigation")
 
+    def _increment_response_stats(self, response: PlaywrightResponse) -> None:
+        stats_prefix = "playwright/response_count"
+        self.stats.inc_value(stats_prefix)
+        self.stats.inc_value(f"{stats_prefix}/resource_type/{response.request.resource_type}")
+        self.stats.inc_value(f"{stats_prefix}/method/{response.request.method}")
+
     def _make_close_browser_context_callback(self, name: str) -> Callable:
         def close_browser_context_callback() -> None:
             logger.debug("Browser context closed: '%s'", name)
@@ -278,3 +290,13 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
             await route.continue_(**overrides)
 
         return _request_handler
+
+
+def _get_response_encoding(headers: Headers, body: str) -> Optional[str]:
+    encoding = None
+    if headers.get("content-type"):
+        content_type = to_unicode(headers["content-type"])
+        encoding = http_content_type_encoding(content_type)
+    if not encoding:
+        encoding = html_body_declared_encoding(body)
+    return encoding
